#include <sys/param.h>
#include <sys/module.h>
#include <sys/kernel.h>
#include <sys/systm.h>
#include <sys/malloc.h>

static void runTest(void);
MALLOC_DECLARE(M_FOO);
MALLOC_DEFINE(M_FOO, "mfoo", "foo mem");

/* The function called at load/unload. */
static int event_handler(struct module *module, int event, void *arg) {
    int e = 0; /* Error, 0 for normal return status */

    switch (event) {
    case MOD_LOAD:
	runTest();
        break;
    case MOD_UNLOAD:
	printf("Done\n");
	uprintf("Done\n");
        break;
    default:
        e = EOPNOTSUPP; /* Error, Operation Not Supported */
        break;
    }

    return(e);
}

static void runTest() {

	const int pageSize = 4096;
        unsigned long size;
        void *ptr = NULL;
	unsigned long long t1, t2;

        for(size = 1; size <= (pageSize -1); size <<= 1) {

		t1 = rdtsc();
                ptr = contigmalloc(size, M_FOO, M_WAITOK, 0, ~0,
                                   pageSize, 0);
		t2 = rdtsc();
		printf("contigmalloc cycles: %llu\n", t2-t1);

                if(ptr == NULL)
                        return;

		t1 = rdtsc();
                contigfree(ptr, size, M_FOO);
		t2 = rdtsc();
		printf("contigfree cycles: %llu\n\n", t2-t1);
		uprintf("Loop complete\n");
        }
}

/* The second argument of DECLARE_MODULE. */
static moduledata_t test_conf = {
    "test",    /* module name */
     event_handler,  /* event handler */
     NULL            /* extra data */
};

DECLARE_MODULE(test, test_conf, SI_SUB_DRIVERS, SI_ORDER_MIDDLE);
